
#    MIDAS Access Server  for  TDRI module 

package require MIDASDomainHandler       ;#  load default support procedures

package require TDRIAccess ;#  load TDRI access library


global MIDAS_TDRIAccessServer_Exists
set MIDAS_TDRIAccessServer_Exists 1

global MIDAS_TDRIAccessServer_Verbose
set MIDAS_TDRIAccessServer_Verbose 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

global MIDAS_RegisterAccessServer_Verbose
set MIDAS_RegisterAccessServer_Verbose 0


upvar #0 TDRIAccessServer_MaxModule MaxModule
if {![info exists MaxModule]} {set MaxModule 4}

upvar #0 TDRIAccessServer_NumModule NumModule
set NumModule 0

upvar #0 TDRIAccessServer_Module Module
set Module 0

upvar #0 TDRIAccessServer_XportList XportList
upvar #0 TDRIAccessServer_Addr Addr
upvar #0 TDRIAccessServer_Port Port


# Use namespaces to isolate your methods
namespace eval urn:TDRIAccessServer {

    proc AccessServerVerbose {option} {
        global MIDAS_TDRIAccessServer_Verbose
        global MIDAS_RegisterAccessServer_Verbose
        global MIDAS_SOAPDomainHandler_Verbose

        set MIDAS_TDRIAccessServer_Verbose $option
        set MIDAS_RegisterAccessServer_Verbose $option
        set MIDAS_SOAPDomainHandler_Verbose $option

        return [rpcvar::rpcvar int 0]
    }

    proc Init {} {
        upvar #0 TDRIAccessServer_NumModule NumModule

        return [rpcvar::rpcvar int $NumModule]
    }

    proc Select {Module} {
        global TDRIAccessServer_Module

        set TDRIAccessServer_Module $Module

        return [rpcvar::rpcvar int 0]
    }

    proc WriteControl {Value} {
        do_WriteControl $Value
        return [rpcvar::rpcvar int 0]
    }

    proc ReadControl {} {
        set vl [do_ReadControl]
        return [rpcvar::rpcvar int $vl]
    }

    proc Status {} {
        set vl [do_Status]
        return [rpcvar::rpcvar int $vl]
    }

    proc Version {} {
        set vl [do_Version]
        return [rpcvar::rpcvar int $vl]
    }


    proc do_WriteControl {Value} {
        upvar #0 MIDAS_TDRIAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method TDRIAccessServer::WriteControl"}
 
#    now start access

        do_Connect

#    do action

        TDRIAccess_WriteControl $Value

        return    ;# completed 
    }

    proc do_ReadControl {} {
        upvar #0 MIDAS_TDRIAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method TDRIAccessServer::ReadControl"}

#    now start access

        do_Connect

#    do action

       set vl [TDRIAccess_ReadControl]

        return $vl   ;# completed 
    }

    proc do_Status {} {
        upvar #0 MIDAS_TDRIAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method TDRIAccessServer::Status"}

#    now start access

        do_Connect

#    do action

       set vl [TDRIAccess_Status]

        return  $vl   ;# completed 
    }

    proc do_Version {} {
        upvar #0 MIDAS_TDRIAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method TDRIAccessServer::Version"}

#    now start access

        do_Connect

#    do action

       set vl [TDRIAccess_Version]

        return  $vl   ;# completed 
    }

proc do_Connect {} {
        upvar #0 MIDAS_TDRIAccessServer_Verbose Verbose
        upvar #0 TDRIAccessServer_Module Module
        upvar #0 TDRIAccessServer_Addr Addr
        upvar #0 TDRIAccessServer_Port Port

            package require TDRIAccess

            TDRIAccess_Verbose $Verbose

            TDRIAccess_Configure -n $Addr($Module) -p $Port($Module)  

            TDRIAccess_Init
}


    # We have to publish the public methods...

     SOAP::export AccessServerVerbose
     SOAP::export Select
     SOAP::export WriteControl
     SOAP::export ReadControl
     SOAP::export Status
     SOAP::export Version

     
}   ;#  end of    namespace eval urn:TDRIAccessServer

# register this service with tclhttpd
set rc [SOAP::Domain::register  -prefix  /TDRIAccessServer -namespace urn:TDRIAccessServer -uri urn:TDRIAccessServer]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/TDRIAccessServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}


package provide TDRIAccessServer 1.0

set msg "$rc Web Service Installed\n"

for {set i 0} {$i < $MaxModule} {incr i} {
    set Addr($i) 0; set Port($i) 0
    if {![info exists XportList($i)]} {continue}

    set Addr($i) [first [split $XportList([set i]) :]]
    set Port($i) [second [split $XportList([set i]) :]]

    append msg "Using TDRI module with Xport at $Addr($i):$Port($i)\n"
    incr NumModule
}

puts "$msg"

   if {[info procs ::DefineMessage] != "::DefineMessage"} {puts "DefineMessage unknown"} else {
       ::DefineMessage -c tdri -l INFORMATION  "$msg"}
